package SPVM::Error::FieldNotSpecified;

1;

=head1 Name

SPVM::Error::FieldNotSpecified - Field Not Specified Errors

=head1 Description

Error::FieldNotSpecified class in L<SPVM> is the error class to indicate a field value is not specifeid.

=head1 Usage

  use Error::FieldNotSpecified;

=head1 Super Class

L<Error|SPVM::Error>

=head1 Copyright & License

Copyright (c) 2025 Yuki Kimoto

MIT License
