package SPVM::TestCase::NativeAPI;



1;

=head1 Name

SPVM::TestCase::NativeAPI - Short Description

=head1 Description

TestCase::NativeAPI class in L<SPVM> has methods to do someting.

=head1 Usage

  use TestCase::NativeAPI;

=head1 Details



=head1 Super Class



=head1 Interfaces



=head1 Enumerations



=head1 Fields



=head1 Class Methods



=head1 Instance Methods



=head1 See Also




=head1 Repository



=head1 Author

[--user-name] C<[--user-email]>

=head1 Copyright & License

Copyright (c) 2025 [--user-name]

MIT License

